use std::{fmt::Display, str::FromStr};

use anyhow::{bail, Result};
use serde::{Deserialize, Serialize};

#[allow(clippy::pedantic)]
#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq, Default)]
pub enum SongType {
    #[default]
    Youtube,
    Spotify,
    Soundcloud,
}

impl Display for SongType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::Soundcloud => write!(f, "SoundCloud")?,
            Self::Spotify => write!(f, "Spotify")?,
            Self::Youtube => write!(f, "YouTube")?,
        }
        Ok(())
    }
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Song {
    url: String,
    typ: SongType
}

#[allow(dead_code)]
impl Song {
    #[allow(clippy::needless_pass_by_value)]
    pub fn from_url_str<S: ToString>(url: S) -> Result<Self> {
        Self::from_url(url::Url::from_str(&url.to_string())?)
    }
    pub fn from_url(url: url::Url) -> Result<Self> {
        Ok(Self {
            url: url.to_string(),
            typ: url.try_into()?,
        })
    }

    pub fn set_type(&mut self, typ: SongType) -> &mut Self {
        self.typ = typ;
        self
    }
    pub fn get_url(&self) -> Result<url::Url> {
        Ok(url::Url::from_str(&self.url)?)
    }
    pub fn get_url_str(&self) -> &String {
        &self.url
    }
    pub fn get_url_str_mut(&mut self) -> &mut String {
        &mut self.url
    }
    pub fn get_type(&self) -> &SongType {
        &self.typ
    }
    pub fn get_type_mut(&mut self) -> &mut SongType {
        &mut self.typ
    }
}




impl TryFrom<url::Url> for SongType {
    type Error = anyhow::Error;

    fn try_from(url: url::Url) -> std::prelude::v1::Result<Self, Self::Error> {
        let Some(host) = url.host_str() else {
            bail!("{url} does not have a host");
        };

        match host {
            "youtube.com" | "youtu.be" | "www.youtube.com" => Ok(Self::Youtube),
            "open.spotify.com"  => Ok(Self::Spotify),
            "SOUNDCLOUD" => Ok(Self::Soundcloud), // TODO: Fix this
            _ => bail!("Unknwon host {url}")
        }
    }
}
