use egui::{Color32, RichText};
use crate::{manifest::song::{Song, SongType}, ui::gui::windows::{self, song_edit::GuiSongEditor, WindowIndex}};

use super::ComponentContextMenu;

pub struct ContextMenu;

pub struct SongInfo {
    pname: String,
    sname: String,
    song: Song,
}

impl SongInfo {
    pub fn new(pname: &str, sname: &str, song: &Song) -> Self {
        Self {
            pname: pname.to_string(),
            sname: sname.to_string(),
            song: song.clone()
        }
    }
    pub fn playlist_name(&self) -> &String {
        &self.pname
    }
    pub fn song_name(&self) -> &String {
        &self.sname
    }
    pub fn song_url(&self) -> &String {
        self.song.get_url_str()
    }
    pub fn song_type(&self) -> &SongType {
        self.song.get_type()
    }
    pub fn song(&self) -> &Song {
        &self.song
    } 
}

impl ComponentContextMenu for ContextMenu {
    type Data = SongInfo;
    fn ui(gui: &mut crate::ui::gui::Gui, ui: &mut egui::Ui, data: &Self::Data) {
        if ui.button("Edit").clicked() {
            let w = gui.windows.get_window::<GuiSongEditor>(WindowIndex::SongEdit);
            w.set_active_song(data.playlist_name(), data.song_name(), data.song_url(), data.song_type());
            gui.windows.open(WindowIndex::SongEdit, true);
            ui.close_menu();
        }

        if ui.button("Download").clicked() {
            if let Err(e) = gui.downloader.download_song_nb(&gui.cfg, data.playlist_name(), data.song_name(), data.song(), gui.manifest.get_format()) {
                log::error!("{e}");
                gui.throw_error(format!("Failed to download song {}: {e}", data.song_name()));
            }
            ui.close_menu();
        }

        if ui.button("Open Source").clicked() {
            if let Err(e) = open::that(data.song_url()) {
                log::error!("{e}");
                gui.throw_error(format!("Failed to open song source: {e}"));
            }
            ui.close_menu();
        }
        if ui.button("Play").clicked() {
            let p = crate::util::get_song_path(data.playlist_name(), data.song_name(), gui.manifest.get_format());
            
            if !p.exists() {
               gui.throw_error("Song does not exist on disk".to_string()); 
            } else if let Err(e) = open::that(p) {
                log::error!("{e}");
                gui.throw_error(format!("Failed to play song: {e}"));
            }
            ui.close_menu();
        }
        if ui.button("Delete from disk").clicked() {
            let p = crate::util::get_song_path(data.playlist_name(), data.song_name(), gui.manifest.get_format());
            if p.exists() {
                if let Err(e) = std::fs::remove_file(p) {
                    gui.throw_error(format!("Failed to delete file: {e}"));
                }
            }
            ui.close_menu();
        }
        if ui.button(RichText::new("Delete").color(Color32::RED)).clicked() {
            let w = gui.windows.get_window::<windows::confirm::ConfirmW>(WindowIndex::Confirm);
            w.set_message(
                "song_list_song_manifest_delete",
                "This will delete the song from the manifest file. This is NOT reversible",
                &[data.playlist_name().clone(), data.song_name().clone()]
            );
            gui.windows.open(WindowIndex::Confirm, true);
            ui.close_menu();
            ui.close_menu();
        }
    }
}
