mod windows;
mod components;

use components::{Component, ComponentUi};
use egui_extras::install_image_loaders;
use windows::{State, WindowIndex, WindowManager};
use crate::{config::ConfigWrapper, downloader::Downloader, manifest::Manifest};


#[derive(Debug, Default)]
pub struct Gui {
    windows: WindowManager,
    manifest: Manifest,
    downloader: Downloader,
    cfg: ConfigWrapper,
    downloading: bool,
    search: components::search_bar::SearchBar,
    current_playlist: String,
}

impl Gui {
    fn new(_: &eframe::CreationContext<'_>, manifest: Manifest, downloader: Downloader, cfg: ConfigWrapper) -> Self {
        Self {
            manifest,
            downloader,
            cfg,
            windows: windows::WindowManager::new(),
            ..Default::default()
        }
    }

    pub fn start(manifest: Manifest, downloader: Downloader, cfg: ConfigWrapper) -> anyhow::Result<()> {
        let native_options = eframe::NativeOptions {
            viewport: egui::ViewportBuilder::default()
                .with_inner_size([400.0, 300.0])
                .with_min_inner_size([300.0, 220.0])
                .with_icon(
                     eframe::icon_data::from_png_bytes(crate::data::APP_ICON_BYTES)?,
                ),
            ..Default::default()
        };

        if let Err(e) = eframe::run_native(
            "McMG",
            native_options,
            Box::new(|cc| Box::new(Gui::new(cc, manifest, downloader, cfg))),
        ) {
            log::error!("Failed to create window: {e}");
        };

        Ok(())
    }

    pub fn throw_error<S: ToString>(&mut self, text: S) {
        let w = self.windows.get_window::<windows::error::GuiError>(WindowIndex::Error);
        w.set_error_message(text);
        self.windows.open(WindowIndex::Error, true);
    }
}

impl eframe::App for Gui {
    fn update(&mut self, ctx: &egui::Context, _: &mut eframe::Frame) {
        install_image_loaders(ctx);
        {
            let mut state = State {
                cfg: self.cfg.clone(),
                downloader: self.downloader.clone(),
                manifest: self.manifest.clone(),
            };
            self.windows.ui(&mut state, ctx);
            self.cfg = state.cfg;
            self.downloader = state.downloader;
            self.manifest = state.manifest;
        }

        components::nav::NavBar::ui(self, ctx);
        egui::CentralPanel::default().show(ctx, |ui| {
            let avail_height = ui.available_height();
            // The central panel the region left after adding TopPanel's and SidePanel's
            //ui.heading(format!("Songs ({})", self.manifest.get_song_count()));
            ui.vertical_centered_justified(|ui| {
                ui.with_layout(egui::Layout::top_down_justified(egui::Align::TOP), |ui| {
                    ui.horizontal(|ui| {
                        ui.set_height(avail_height);
                        components::side_nav::SideNav::ui(self, ui);
                        components::song_list::SongList::ui(self, ui);
                    });
                    ui.separator();
                    
                    ui.with_layout(egui::Layout::bottom_up(egui::Align::LEFT), |ui| {
                        egui::warn_if_debug_build(ui);
                    });
                });
            });
        });
        // Make sure we dont wait for any updates cause we depend on the gui code for downloads
        ctx.request_repaint();
    }
}
