use std::collections::HashMap;
use crate::{config::ConfigWrapper, downloader::Downloader, manifest::Manifest};

pub mod song_edit;
pub mod error;
pub mod import_playlist;
pub mod song_new;
pub mod confirm;

pub trait Window: std::fmt::Debug {
    fn ui(&mut self, state: &mut State, ctx: &egui::Context, open: &mut bool) -> anyhow::Result<()>;
}

#[derive(Debug, Hash, Eq, PartialEq, Clone, Copy)]
pub enum WindowIndex {
    Error,
    ImportPlaylist,
    SongEdit,
    SongNew,
    Confirm
}


#[derive(Debug,Default)]
pub struct WindowManager {
    opened: HashMap<WindowIndex, bool>,
    windows: HashMap<WindowIndex, Box<dyn Window>>
}

pub struct State {
    pub downloader: Downloader,
    pub manifest: Manifest,
    pub cfg: ConfigWrapper,
}

impl WindowManager {
    pub fn new() -> Self {
        let mut windows: HashMap<WindowIndex, Box<dyn Window>> = HashMap::new();
        windows.insert(WindowIndex::Error, Box::<error::GuiError>::default());
        windows.insert(WindowIndex::ImportPlaylist, Box::<import_playlist::GuiImportPlaylist>::default());
        windows.insert(WindowIndex::SongEdit, Box::<song_edit::GuiSongEditor>::default());
        windows.insert(WindowIndex::SongNew, Box::<song_new::GuiNewSong>::default());
        windows.insert(WindowIndex::Confirm, Box::<confirm::ConfirmW>::default());
        Self {
            windows,
            ..Default::default()
        }
    }
    
    #[allow(dead_code)]
    pub fn is_open(&self, id: WindowIndex) -> bool {
        *self.opened.get(&id).unwrap()
    }

    pub fn open(&mut self, id: WindowIndex, open: bool) {
        self.opened.insert(id, open);
    }

    pub fn ui(&mut self, state: &mut State, ctx: &egui::Context) {
        for (id, window) in &mut self.windows {
            if !self.opened.contains_key(id) {
                self.opened.insert(*id, false);
            }
            let open = self.opened.get_mut(id).unwrap();
            if let Err(e) = window.ui(state, ctx, open) {
                log::error!("Window {id:?} errored: {e}");
            }
        }
    }

    pub fn get_window<T: Window + 'static>(&mut self, id: WindowIndex) -> &mut Box<T> {
        let w = self.windows.get_mut(&id).unwrap();
        unsafe {
            crate::util::as_any_mut(w).downcast_mut_unchecked()
        }
    }
}
